#!/usr/bin/env python
# -*- coding: utf-8 -*-

# http://en.wikipedia.org/wiki/Electronic_color_code
# http://www.okaphone.nl/calc/resistor.shtml
# http://www.pygtk.org/pygtk2tutorial/index.html
# http://faq.pygtk.org/index.py?req=show&file=faq22.002.htp

#CVT: PYTHONPATH=/usr/local/lib/python2.4/site-packages python2.4 pycore.py
import pygtk
pygtk.require('2.0')
import gtk

import libresistor

import locale
import gettext
APP = 'PyCoRe'
DIR = 'locale'
locale.setlocale(locale.LC_ALL)
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
_ = gettext.gettext

class PyCoRe:
	#formatString = '<span size="xx-large"><b>%s</b></span>'
	resistor = libresistor.Resistor()
	
	def about(self, widget):
		about = gtk.AboutDialog()
		about.set_name(APP)
		about.set_comments(_("Resistors color code"))
		about.set_authors(["Bc. Jan Kaláb"])
		about.set_documenters(["Bc. Jan Kaláb"])
		about.set_artists(["Gerald_G"])
		about.set_translator_credits("Bc. Jan Kaláb, Peter Krempa")
		about.set_logo(self.icon)
		about.set_icon(self.icon)
		about.run()
		about.destroy()
	
	def getComboValue(self, combo):
		return combo.get_model()[combo.get_active()][0]
	
	def change(self, widget):
		if widget is self.valueEntry or widget is self.tolEntry:
			#print "Entry has changed to " + widget.get_text()
			val = self.valueEntry.get_text()
			tol = self.tolEntry.get_text()
			self.resistor.setValue(val, tol)
			self.val1ComboBox.handler_block(self.val1Change)
			self.val2ComboBox.handler_block(self.val2Change)
			self.mulComboBox.handler_block(self.mulChange)
			if len(val) == 1:
				self.val1ComboBox.set_active(0)
				self.val2ComboBox.set_active(int(val[0]))
				self.mulComboBox.set_active(0)
			elif len(val) > 1:
				self.val1ComboBox.set_active(int(val[0]))
				self.val2ComboBox.set_active(int(val[1]))
				self.mulComboBox.set_active(len(val) - 2)
			self.val1ComboBox.handler_unblock(self.val1Change)
			self.val2ComboBox.handler_unblock(self.val2Change)
			self.mulComboBox.handler_unblock(self.mulChange)
		else:
			val1 = self.resistor.colors.index(self.getComboValue(self.val1ComboBox))
			val2 = self.resistor.colors.index(self.getComboValue(self.val2ComboBox))
			mul = self.resistor.colors.index(self.getComboValue(self.mulComboBox))
			tol = self.resistor.colors.index(self.getComboValue(self.tolComboBox))
			self.resistor.setValue((val1 * 10 + val2) * 10**mul)
			self.resistor.setValue(tol = self.resistor.tolerances[self.getComboValue(self.tolComboBox)])
			self.valueEntry.handler_block(self.valueChange)
			self.tolEntry.handler_block(self.tolEntryChange)
			self.valueEntry.set_text(str(self.resistor))
			self.tolEntry.set_text(str(self.resistor.getValue()[1]))
			self.valueEntry.handler_unblock(self.valueChange)
			self.tolEntry.handler_unblock(self.tolEntryChange)
		print repr(self.resistor)
	
	def __init__(self):
		self.icon = gtk.gdk.pixbuf_new_from_file('res/icon.svg')
		
		#Main window
		window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		window.set_position(gtk.WIN_POS_CENTER)
		window.connect("destroy", gtk.main_quit)
		window.set_title(APP)
		window.set_icon(self.icon)
		
		#Accelerators
		acc = gtk.AccelGroup()
		window.add_accel_group(acc)
		
		#Vertical box
		vbox = gtk.VBox(False, 1)
		window.add(vbox)
		
		#Menu
		menu = gtk.MenuBar()
		fileMenu = gtk.Menu()
		helpMenu = gtk.Menu()
		quitMenuItem = gtk.MenuItem(_("_Quit"))
		fileMenuItem = gtk.MenuItem(_("_File"))
		aboutMenuItem = gtk.MenuItem(_("_About"))
		helpMenuItem = gtk.MenuItem(_("_Help"))
		key, mod = gtk.accelerator_parse("<Alt>X")
		quitMenuItem.add_accelerator("activate", acc, key, mod, gtk.ACCEL_VISIBLE)
		key, mod = gtk.accelerator_parse("F1")
		aboutMenuItem.add_accelerator("activate", acc, key, mod, gtk.ACCEL_VISIBLE)
		quitMenuItem.connect("activate", gtk.main_quit)
		aboutMenuItem.connect("activate", self.about)
		fileMenu.append(quitMenuItem)
		helpMenu.append(aboutMenuItem)
		fileMenuItem.set_submenu(fileMenu)
		helpMenuItem.set_submenu(helpMenu)
		menu.append(fileMenuItem)
		menu.append(helpMenuItem)
		vbox.pack_start(menu, False)
		
		#Table
		table = gtk.Table(3, 4, True)
		table.set_border_width(5)
		vbox.pack_start(table)
		
		#Labels
		table.attach(gtk.Label(_("First digit")), 0, 1, 0, 1)
		table.attach(gtk.Label(_("Second digit")), 1, 2, 0, 1)
		table.attach(gtk.Label(_("Multiplier")), 2, 3, 0, 1)
		table.attach(gtk.Label(_("Tolerance")), 3, 4, 0, 1)
		
		#Combo boxes
		colorList = gtk.ListStore(str)
		for color in self.resistor.colors:
			colorList.append([color])
		self.val1ComboBox = gtk.ComboBox(colorList)
		self.val2ComboBox = gtk.ComboBox(colorList)
		self.mulComboBox = gtk.ComboBox(colorList)
		self.tolComboBox = gtk.ComboBox(colorList)
		table.attach(self.val1ComboBox, 0, 1, 1, 2)
		table.attach(self.val2ComboBox, 1, 2, 1, 2)
		table.attach(self.mulComboBox, 2, 3, 1, 2)
		table.attach(self.tolComboBox, 3, 4, 1, 2)
		cell = gtk.CellRendererText()
		self.val1ComboBox.pack_start(cell)
		self.val2ComboBox.pack_start(cell)
		self.mulComboBox.pack_start(cell)
		self.tolComboBox.pack_start(cell)
		self.val1ComboBox.set_attributes(cell, cell_background = 0)
		self.val2ComboBox.set_attributes(cell, cell_background = 0)
		self.mulComboBox.set_attributes(cell, cell_background = 0)
		self.tolComboBox.set_attributes(cell, cell_background = 0)
		self.val1ComboBox.set_active(0)
		self.val2ComboBox.set_active(0)
		self.mulComboBox.set_active(0)
		self.tolComboBox.set_active(0)
		
		#Value entry
		self.valueEntry = gtk.Entry()
		self.valueEntry.set_alignment(0.5)
		self.valueEntry.set_text(str(self.resistor))
		table.attach(self.valueEntry, 0, 3, 2, 3)
		
		#Tolerance entry
		self.tolEntry = gtk.Entry()
		self.tolEntry.set_alignment(0.5)
		self.tolEntry.set_text(str(self.resistor.tolerance))
		table.attach(self.tolEntry, 3, 4, 2, 3)
		
		#Callbacky
		self.val1Change = self.val1ComboBox.connect('changed', self.change)
		self.val2Change = self.val2ComboBox.connect('changed', self.change)
		self.mulChange = self.mulComboBox.connect('changed', self.change)
		self.tolChange = self.tolComboBox.connect('changed', self.change)
		self.valueChange = self.valueEntry.connect('changed', self.change)
		self.tolEntryChange = self.tolEntry.connect('changed', self.change)
		
		window.show_all()
	
	def main(self):
		gtk.main()

if __name__ == "__main__":
	pycore = PyCoRe()
	pycore.main()
